@@ -198,7 +198,7 @@ class DexcellServiceMessage(object):
         timeformat = "%Y/%m/%d %H:%M"
         outPut = "DexcellServiceMessage(node=%s" % str(self.node)
         outPut += ", service=%s" % str(self.service)
-        outPut += ", timestamp=%s" % time.strftime(timeformat, self.timestamp)
+        outPut += ", timestamp=%s" % self.timestamp.isoformat()
         outPut += ", value=%s" % str(self.value)
         outPut += ", seqnum=%s)" % str(self.seqnum)
         return outPut
@@ -227,7 +227,6 @@ class DexcellServiceMessage(object):
 class DexcellSender(object):
 
     DEFAULT_SERVER = 'insert.dexcell.com'
-    DEFAULT_URL = '/insert-json.htm'
     DEFAULT_LOGFILE = '/var/log/dexma/DexcellSender.log'
     DEFAULT_LOGLEVEL = logging.INFO
     DEFAULT_GATEWAY = 'None'
@@ -235,11 +234,11 @@ class DexcellSender(object):
 
     def __init__(self, gateway=DEFAULT_GATEWAY, loggerName=DEFAULT_LOGGERNAME,
                  logfile=DEFAULT_LOGFILE, loglevel=DEFAULT_LOGLEVEL,
-                 server=DEFAULT_SERVER, url=DEFAULT_URL,
+                 server=DEFAULT_SERVER,
                  https=True, timeout=30.0):
         self.__https = https
         self.__server = server
-        self.__url = url
+        self.__url = self.__server+'/'+gateway+'/insert-json.html'
         self.__timeout = timeout
         self.__gateway = gateway
         self.__logger = logging.getLogger(loggerName)
@@ -252,11 +251,11 @@ class DexcellSender(object):
 
     def setup(self, gateway=DEFAULT_GATEWAY, loggerName=DEFAULT_LOGGERNAME,
               logfile=DEFAULT_LOGFILE, loglevel=DEFAULT_LOGLEVEL,
-              server=DEFAULT_SERVER, url=DEFAULT_URL):
+              server=DEFAULT_SERVER):
         """Setup the Dexcell Sender Object
         """
         self.__server = server
-        self.__url = url
+        self.__url = self.__server+'/'+gateway+'/insert-json.html'
         self.__gateway = gateway
         self.__logger = logging.getLogger(loggerName)
         if len(self.__logger.handlers) == 0:
@@ -271,12 +270,60 @@ class DexcellSender(object):
         """
         self.__gateway = gateway
 
-    def __insertRawJSONData(self, data, maxerrorRetry):
+    def _get_token(self):
+        return 'Gazpacho y Mochilo... siempre van con Pincho....'
+
+    def pack_service_messages(self,pack):
+        groups = {}
+        total_service = 0
+        for reading in pack:
+            nnid = None
+            snid = None
+            try:
+                nnid = reading['nodeNetworkId']
+            except KeyError:
+                # we need to raise to enable error management in previous function
+                try:
+                    nnid = reading['nodeId']
+                except:
+                    pass
+            try:
+                snid = reading['serviceNetworkId']
+            except KeyError:
+                try:
+                    snid = int(reading['serviceId'])
+                except Exception as e:
+                    raise e
+            except Exception as e:
+                raise e
+            # this is the key to order and merge
+            key = nnid + '#' + reading['timeStamp']
+            try:
+                groups[key]['seqnum'] = int(reading['seqNum'])
+            except KeyError:
+                groups[key] = {}
+                groups[key]['seqnum'] = int(reading['seqNum'])
+                groups[key]['ts'] = reading['timeStamp']
+                groups[key]['did'] = nnid
+            try:
+                dic = {
+                        'p' : snid,
+                        'v': float(reading['value'])
+                }
+                groups[key]['values'].append(dic)
+            except KeyError:
+                groups[key]['values'] = {}
+                groups[key]['values'][snid] = float(reading['value'])
+            total_service = total_service + 1
+        return groups.values(), total_service
+
+    def __insertRawJSONData(self, data, maxerrorRetry,token):
         """Insert the raw data string to the server
         """
-        params = 'data=' + data
-        headers = {"Content-type": "application/x-www-form-urlencoded",
-                   "Accept": "text/plain"}
+        params = data
+        headers = {"Content-type": "application/json;charset=utf-8",
+                   "Accept": "text/plain",
+                   'x-dexcell-source-token': token}
         if self.__https:
             conn = httplib.HTTPSConnection(self.__server,
                                            timeout=self.__timeout)
@@ -313,15 +360,13 @@ class DexcellSender(object):
             'serviceNetworkId': int(serviceMessage.service),
             'value': float(serviceMessage.value),
             'seqNum': int(serviceMessage.seqnum),
-            'timeStamp': time.strftime("%Y-%m-%dT%H:%M:%S.000 " + timezone,
-                                       serviceMessage.timestamp)
-        }
-        data = {
-            'gatewayId': self.__gateway,
-            'service': [reading]
+            'timeStamp': serviceMessage.timestamp
         }
+        data = {}
+        data['msg'], total_service = self.pack_service_messages(reading)
         for key in extraparams.keys():
             data[key] = extraparams[key]
+        token = self._get_token()
         result = self.__insertRawJSONData(json.dumps(data),maxerror)
         return result
 
@@ -336,16 +381,14 @@ class DexcellSender(object):
                 'serviceNetworkId': int(serviceMessage.service),
                 'value': float(serviceMessage.value),
                 'seqNum': int(serviceMessage.seqnum),
-                'timeStamp': time.strftime("%Y-%m-%dT%H:%M:%S.000 " + timezone,
-                                           serviceMessage.timestamp)
+                'timeStamp': serviceMessage.timestamp
             }
             readings.append(reading)
-        data = {
-            'gatewayId': self.__gateway,
-            'service': readings
-        }
+        data = {}
+        data['msg'], total_service = self.pack_service_messages(readings)
         for key in extraparams.keys():
             data[key] = extraparams[key]
+        token = self._get_token()
         result = self.__insertRawJSONData(json.dumps(data),maxerror)
         return result
 
@@ -370,6 +413,7 @@ class MultiPartForm(object):
     def add_file(self, fieldname, filename, fileHandle, mimetype=None):
         """Add a file to be uploaded."""
         body = fileHandle.read()
+        #body = fileHandle.getvalue()
         if mimetype is None:
             mimetype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
         self.files.append((fieldname, filename, mimetype, body))
@@ -634,6 +678,15 @@ class DexcellRestApi(object):
         data = json.loads(data, object_hook=self._datetime_parser)
         return data
 
+    def get_deployment_gateways(self, dep_id):
+        """ return array with parameters {id, name, networkid}
+            from deployment number dep_id
+        """
+        url = "/deployments/%i/gateways.json" % dep_id
+        device_list = self._call_rest(url)
+        return device_list
+
+
     def get_location(self, loc_id):
         """ return dict with basic information from locat"""
         url = "/locations/%i.json" % loc_id
@@ -802,11 +855,18 @@ class DexcellRestApi(object):
         except KeyError:
             return []
 
-    def upload_report_file(self, id, path):
-        form = MultiPartForm()
-        form.add_file('report_file', 'report.pdf', open(path, 'r'))
+    def get_mvp_info(self, mvp_id):
+        """ return array of mvp information for a concret id
+        """
+        url = "/mvp/%i.json" % mvp_id
+        device_list = self._call_rest(url)
+        return device_list
 
-        url = self.endpoint + "reportfiles/file/" + str(id)
+    def upload_report_file(self, id, path, type_report):
+        form = MultiPartForm()
+        form.add_file('report_file', 'report.%s' % type_report, open(path, 'r'))
+
+        url = self.endpoint + "reportfiles/file/" + str(id) + "?extension=" + type_report
 
         req = urllib2.Request(url, headers={'x-dexcell-token': self.token})
         req.add_header("Content-type", form.get_content_type())
@@ -817,8 +877,24 @@ class DexcellRestApi(object):
         result = json.loads(data)
         return result
 
+    def upload_report_file_test(self, id, report, type_report):
+        form = MultiPartForm()
+        form.add_file('report_file', 'report.%s' % type_report, report)
+
+        url = self.endpoint + "reportfiles/file/" + str(id) + "?extension=" + type_report
+
+        req = urllib2.Request(url, headers={'x-dexcell-token': self.token})
+        req.add_header("Content-type", "multipart/form-data")
+        req.add_data(str(form))
+        self.logger.info('getting pending report files')
+        response = urllib2.urlopen(req)
+        data = response.read()
+        result = json.loads(data)
+        report.close()
+        return result
+
     def set_status_report_file(self, id, status):
         url = "reportfiles/file/" + str(id) + "/status?status=" + str(status)
         payload = "status=" + str(status)
         result = self._call_rest(url)
-        return result
+        return result
\ No newline at end of file
